/**
 * @file cmdbuf.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * A buffer for terminal stuff
 *
 * BE CAREFUL - THIS IS NOT READY FOR MULTITHREADING (!)
**/
#include "cmdbuf.h"
#include <stdio.h>
#include <stdarg.h>

/* main data array */
char cmdbuf[CMDBUF_BUF_LINES][CMDBUF_BUF_SIZE];
unsigned int curline = 0;



/**
 * cmdbuf_printf()
 * Print something to the internal console
 * Only one line! Do not use \n or \r
**/
void cmdbuf_printf(const char* format, ...)
{
	va_list argptr;
	va_start(argptr, format);
	/* print to line string */
	vsnprintf(cmdbuf[curline], CMDBUF_BUF_SIZE, format, argptr);
	/* ensure string ends */
	cmdbuf[curline][CMDBUF_BUF_SIZE-1] = '\0';
	va_end(argptr);
	/* goto next line */
	curline = (curline + 1) % CMDBUF_BUF_LINES;
}



/**
 * cmdbuf_getline()
 * Get a line from console
 * @return Line string
**/
const char* cmdbuf_getline(unsigned int n)
{
	if(n >= CMDBUF_BUF_LINES)
		return "LINE ERROR - LINE DOES NOT EXIST!";
	return cmdbuf[(curline + n)%CMDBUF_BUF_LINES];
}



/**
 * cmdbuf_clear()
 * Clear internal console screen
**/
void cmdbuf_clear()
{
	unsigned int n;
	for(n=0; n<CMDBUF_BUF_LINES; n++)
		cmdbuf[n][0] = '\0';
	curline = 0;
}
