/**
 * @file programmer.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Jan-2013 / All rights reserved. 
 * ---------------------------------------------------------
 * Provides gui content
**/
#include "programmer.h"
#include "listing.h"
#include "dmxgui_tools.h"
#include "../dmxplayer.h"
#include <stdlib.h>

int channel=0;
int channel_cnt=0;
int input_cnt=0;
char input[5];


void programmer_init(int _channel_cnt)
{
	channel_cnt = _channel_cnt;
	input[0] = '\0';
	input_cnt = 0;
	channel = 0;
}


/* A key event in active play mode */
void programmer_keyevent(int c)
{
	int tmp;
	if( (c >= '0') && (c <= '9') )
	{
		if(input_cnt < 3)
		{
			input[input_cnt]   = c;
			input[++input_cnt] = '\0';
		}
	}
	else
	{
		input_cnt=0;
		switch(c)
		{	
			/* toogle channel */
			case KEY_LEFT:
				channel--;
				if(channel < 0) channel = 0;
				break;
			case KEY_RIGHT:
				channel++;
				if(channel >= channel_cnt) channel = channel_cnt - 1;
				break;
			case KEY_UP:
				channel -= (COLS-3)/5;
				if(channel < 0) channel = 0;
				break;
			case KEY_DOWN:
				channel += (COLS-3)/5;
				if(channel >= channel_cnt) channel = channel_cnt - 1;
				break;
			/* toogle first channel */
			case KEY_HOME:
				channel = 0;
				break;
			/* toogle last channel */
			case KEY_END:
				channel = channel_cnt - 1;
				break;
			/* unset channel */
			case KEY_F(5):
			case 'U':
				dmxplayer_unset_prog(channel);
				break;
			/* clear programmer */
			case KEY_F(6):
			case 'C':
				dmxplayer_clear_prog();
				break;
			/* STORE */
			case 'S':
				tmp = listing_getSelectedProgram();
				dmxplayer_store_program(tmp);
				break;
			/* ENTER -> Use input as value */
			case KEY_ENTER:
			case '\n':
			case '\r':
				if(input[0]=='\0') break;
				dmxplayer_set_prog(channel, atoi(input));
				input[0] = '\0';
				break;
			/* +/- 1 */
			case '+':					
				dmxplayer_set_prog_add(channel, 1);
				break;
			case '-':
				dmxplayer_set_prog_add(channel, -1);
				break;
			/* +/- 10 */
			case KEY_PPAGE:
				dmxplayer_set_prog_add(channel, 10);
				break;
			case KEY_NPAGE:
				dmxplayer_set_prog_add(channel, -10);
				break;
		}
	}
}


/**
 * Print programmer window 
 * @param wnd          Window
 * @param channel      Selected DMX channel
 * @param channel_cnt  Number of channels
**/
void programmer_printer(WINDOW* wnd)
{
	int x, y, i, val, selprog;	
	dmxprogram_t *p;
	x = 2;
	y = 5;
	selprog = listing_getSelectedProgram();
	p = dmxplayer_getprogram(selprog);

	/* draw window frame */
	wbkgd(wnd, COLOR_PAIR(1));
	box(wnd, 0, 0);
	if(p != 0) mvwprintw(wnd, 3, 2, "Recording into [%d] \'%s\' - %d steps stored", selprog+1, p->name, p->size);

	/* draw window content */
	wattron(wnd, COLOR_PAIR(2));
	for(i=0; i<channel_cnt; ++i)
	{	
		/* selected channel */
		if(channel == i)
		{
			wattron(wnd, A_REVERSE); 
				mvwprintw(wnd, y, x, " %3d ", i+1);
				if(input_cnt == 0)
				{
					val = dmxplayer_get_prog_value(i);
					if(val>=0) mvwprintw(wnd, y+1, x, " %03d ", val);
					else      mvwprintw(wnd, y+1, x, " --- ");
				}
				else
				{					
					wattroff(wnd, COLOR_PAIR(2)); wattron(wnd, COLOR_PAIR(4));
						mvwprintw(wnd, y+1, x, ">%03d<", atoi(input));
					wattroff(wnd, COLOR_PAIR(4)); wattron(wnd, COLOR_PAIR(2));
				}
			wattroff(wnd, A_REVERSE);
		}
		/* other channels */
		else
		{
			mvwprintw(wnd, y, x, " %3d ", i+1);
			val = dmxplayer_get_prog_value(i);
			if(val>=0) mvwprintw(wnd, y+1, x, " %03d ", val);
			else      mvwprintw(wnd, y+1, x, " --- ");
		}
		/* calculate next position */
		x += 5;
		if( (x+5) >= COLS) 
		{
			x = 2;
			y += 3;
		}
	}
	wattroff(wnd, COLOR_PAIR(2));
	/* show information */	
	dmxgui_print_key_bindings(wnd);
}


